%{
/*
 * lang.y - Input Wheel language
 *
 * Written 2007 by Werner Almesberger
 * Copyright 2007 Werner Almesberger
 */


#include "draw.h"
#include "wheel.h"
#include "layout.h"
#include "action.h"
#include "pattern.h"
#include "map.h"


static int zone, pos;

%}


%union {
    int num;
    const char *str;
    struct {
	int radius;
	int angle;
    } polar;
    struct action *act;
};

%token		TOK_ACCEPT TOK_CENTER TOK_EXIT TOK_FIELD TOK_LAYOUT TOK_MAP
%token		TOK_NEUTRAL TOK_POLAR TOK_RADIUS TOK_RESET TOK_SEGMENTS
%token		TOK_SEND TOK_SIZE TOK_SPOKES TOK_TR TOK_WHEEL TOK_ZERO
%token		TOK_ARROW
%token	<num>	NUM
%token	<str>	ID STRING

%type	<num>	spokes opt_zero radius size
%type	<polar>	location
%type	<act>	action command commands
%type	<act>	accept_cmd exit_cmd reset_cmd map_cmd layout_cmd send_cmd
%type	<act>	tr_cmd

%%

all:
    shape
    content
    ;

shape:
    TOK_WHEEL '{' spokes opt_zero
	{
	    wheel_init($3, $4);
	    zone_init($3+1);
	}
      center segments '}' opt_semi
    ;

spokes:
    TOK_SPOKES NUM ';'
	{
	    $$ = $2;
	}
    ;

opt_zero:
	{
	    $$ = 0;
	}
    | TOK_ZERO NUM ';'
	{
	    $$ = $2;
	}
    ;

center:
    | TOK_CENTER ID '{' opt_neutral radius
	{
	    zone_name($2);
	    wheel_center($5);
	}
      fields '}' opt_semi
    ;

opt_neutral:
    | TOK_NEUTRAL ';'
	{
	    neutral_center = 1;
	}
    ;

radius:
    TOK_RADIUS NUM ';'
	{
	    $$ = $2;
	}
    ;


fields:
    field
    | fields field
    ;

field:
    TOK_FIELD '{' size location '}' opt_semi
	{
	    wheel_field($3, $4.radius, $4.angle);
	}
    ;

size:
    TOK_SIZE NUM ';'
	{
	    $$ = $2;
	}
    ;

location:
    TOK_POLAR NUM ',' NUM ';'
	{
	    $$.radius = $2;
	    $$.angle = $4;
	}
    ;

segments:
    TOK_SEGMENTS ids '{' radius
	{
	    wheel_segment($4);
	}
      fields '}' opt_semi
    ;

ids:
    ID
	{
	    zone_name($1);
	}
    | ids ',' ID
	{
	    zone_name($3);
	}
    ;

content:
    content_item
    | content content_item
    ;

content_item:
    layout
    | map
    ;

layout:
    TOK_LAYOUT STRING
	{
	    begin_draw(0);
	    wheel_draw();
	}
     '{' labels '}' opt_semi
	{
	    layout_store($2, end_draw());
	}
    ;

labels:
    | labels label
    ;

label:
    ID '='
	{
	    zone = find_zone($1);
	}
      strings ';'
    ;

strings:
	{
	    pos = 0;
	}
    | STRING
	{
	    pos = 0;
	    wheel_label(zone, pos, $1);
	}
    | strings ','
	{
	    pos++;
	}
    | strings ',' STRING
	{
	    wheel_label(zone, ++pos, $3);
	}
    ;

map:
    TOK_MAP STRING '{' gestures '}' opt_semi
	{
	    map_new($2);
	    pattern_clear();
	}
    ;

gestures:
    | gestures gesture
    ;

gesture:
    sequence TOK_ARROW action
	{
	    pattern_action($3);
	}
    ;

sequence:
    ID
	{
	    pattern_start();
	    pattern_enter(find_zone($1));
	    free((void *) $1);
	}
    | sequence '-' ID
	{
	    pattern_enter(find_zone($3));
	    free((void *) $3);
	}
    ;

action:
    ';'
	{
	    $$ = NULL;
	}
    | command
	{
	    $$ = $1;
	}
    | '{' commands '}' opt_semi
	{
	    $$ = $2;
	}
    ;

commands:
	{
	    $$ = NULL;
	}
    | command commands
	{
	    $1->next = $2;
	    $$ = $1;
	}
    ;

command:
    accept_cmd
	{
	    $$ = $1;
	}
    | exit_cmd
	{
	    $$ = $1;
	}
    | layout_cmd
	{
	    $$ = $1;
	}
    | map_cmd
	{
	    $$ = $1;
	}
    | reset_cmd
	{
	    $$ = $1;
	}
    | send_cmd
	{
	    $$ = $1;
	}
    | tr_cmd
	{
	    $$ = $1;
	}
    ;

accept_cmd:
    TOK_ACCEPT ';'
	{
	    $$ = new_action(action_accept);
	}
    ;

exit_cmd:
    TOK_EXIT ';'
	{
	    $$ = new_action(action_exit);
	}
    ;

layout_cmd:
    TOK_LAYOUT STRING ';'
	{
	    $$ = new_action(action_layout);
	    $$->s1 = $2;
	}
    ;

map_cmd:
    TOK_MAP STRING ';'
	{
	    $$ = new_action(action_map);
	    $$->s1 = $2;
	}
    ;

reset_cmd:
    TOK_RESET ';'
	{
	    $$ = new_action(action_reset);
	}
    ;

send_cmd:
    TOK_SEND STRING ';'
	{
	    $$ = new_action(action_send);
	    $$->s1 = $2;
	}
    ;

tr_cmd:
    TOK_TR STRING STRING ';'
	{
	    $$ = new_action(action_tr);
	    $$->s1 = $2;
	    $$->s2 = $3;
	}
    ;

opt_semi:
    | ';'
    ;
